@@ -24,6 +24,7 @@
 
 pub mod asyncjob;
 mod blame;
+mod branches;
 pub mod cached;
 mod commit_files;
 mod diff;
@@ -39,9 +40,11 @@ mod revlog;
 mod status;
 pub mod sync;
 mod tags;
+mod treefiles;
 
 pub use crate::{
 	blame::{AsyncBlame, BlameParams},
+	branches::AsyncBranchesJob,
 	commit_files::{AsyncCommitFiles, CommitFilesParams},
 	diff::{AsyncDiff, DiffParams, DiffType},
 	error::{Error, Result},
@@ -59,6 +62,7 @@ pub use crate::{
 		status::{StatusItem, StatusItemType},
 	},
 	tags::AsyncTags,
+	treefiles::AsyncTreeFilesJob,
 };
 pub use git2::message_prettify;
 use std::{
@@ -67,7 +71,7 @@ use std::{
 };
 
 /// this type is used to communicate events back through the channel
-#[derive(Copy, Clone, Debug, PartialEq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub enum AsyncGitNotification {
 	/// this indicates that no new state was fetched but that a async process finished
 	FinishUnchanged,
@@ -78,6 +82,8 @@ pub enum AsyncGitNotification {
 	///
 	Log,
 	///
+	FileLog,
+	///
 	CommitFiles,
 	///
 	Tags,
@@ -93,6 +99,10 @@ pub enum AsyncGitNotification {
 	RemoteTags,
 	///
 	Fetch,
+	///
+	Branches,
+	///
+	TreeFiles,
 }
 
 /// helper function to calculate the hash of an arbitrary type that implements the `Hash` trait
