@@ -1,11 +1,14 @@
-use super::{utils::repo, CommitId};
-use crate::{error::Result, sync::utils::get_head_repo};
+use super::{CommitId, RepoPath};
+use crate::{
+	error::Result,
+	sync::{repository::repo, utils::get_head_repo},
+};
 use git2::{ErrorCode, ObjectType, Repository, Signature};
 use scopetime::scope_time;
 
 ///
 pub fn amend(
-	repo_path: &str,
+	repo_path: &RepoPath,
 	id: CommitId,
 	msg: &str,
 ) -> Result<CommitId> {
@@ -57,8 +60,8 @@ pub(crate) fn signature_allow_undefined_name(
 	signature
 }
 
-/// this does not run any git hooks
-pub fn commit(repo_path: &str, msg: &str) -> Result<CommitId> {
+/// this does not run any git hooks, git-hooks have to be executed manually, checkout `hooks_commit_msg` for example
+pub fn commit(repo_path: &RepoPath, msg: &str) -> Result<CommitId> {
 	scope_time!("commit");
 
 	let repo = repo(repo_path)?;
@@ -92,27 +95,36 @@ pub fn commit(repo_path: &str, msg: &str) -> Result<CommitId> {
 ///
 /// This function will return an `Err(…)` variant if the tag’s name is refused
 /// by git or if the tag already exists.
-pub fn tag(
-	repo_path: &str,
+pub fn tag_commit(
+	repo_path: &RepoPath,
 	commit_id: &CommitId,
 	tag: &str,
+	message: Option<&str>,
 ) -> Result<CommitId> {
-	scope_time!("tag");
+	scope_time!("tag_commit");
 
 	let repo = repo(repo_path)?;
 
-	let signature = signature_allow_undefined_name(&repo)?;
 	let object_id = commit_id.get_oid();
 	let target =
 		repo.find_object(object_id, Some(ObjectType::Commit))?;
 
-	Ok(repo.tag(tag, &target, &signature, "", false)?.into())
+	let c = if let Some(message) = message {
+		let signature = signature_allow_undefined_name(&repo)?;
+		repo.tag(tag, &target, &signature, message, false)?.into()
+	} else {
+		repo.tag_lightweight(tag, &target, false)?.into()
+	};
+
+	Ok(c)
 }
 
 #[cfg(test)]
 mod tests {
 
 	use crate::error::Result;
+	use crate::sync::tags::Tag;
+	use crate::sync::RepoPath;
 	use crate::sync::{
 		commit, get_commit_details, get_commit_files, stage_add_file,
 		tags::get_tags,
@@ -120,13 +132,13 @@ mod tests {
 		utils::get_head,
 		LogWalker,
 	};
-	use commit::{amend, tag};
+	use commit::{amend, tag_commit};
 	use git2::Repository;
 	use std::{fs::File, io::Write, path::Path};
 
 	fn count_commits(repo: &Repository, max: usize) -> usize {
 		let mut items = Vec::new();
-		let mut walk = LogWalker::new(&repo, max).unwrap();
+		let mut walk = LogWalker::new(repo, max).unwrap();
 		walk.read(&mut items).unwrap();
 		items.len()
 	}
@@ -136,7 +148,8 @@ mod tests {
 		let file_path = Path::new("foo");
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		File::create(&root.join(file_path))
 			.unwrap()
@@ -159,7 +172,8 @@ mod tests {
 		let file_path = Path::new("foo");
 		let (_td, repo) = repo_init_empty().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		assert_eq!(get_statuses(repo_path), (0, 0));
 
@@ -185,7 +199,8 @@ mod tests {
 		let file_path2 = Path::new("foo2");
 		let (_td, repo) = repo_init_empty()?;
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		File::create(&root.join(file_path1))?.write_all(b"test1")?;
 
@@ -221,7 +236,8 @@ mod tests {
 		let file_path = Path::new("foo");
 		let (_td, repo) = repo_init_empty().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		File::create(&root.join(file_path))?
 			.write_all(b"test\nfoo")?;
@@ -230,25 +246,56 @@ mod tests {
 
 		let new_id = commit(repo_path, "commit msg")?;
 
-		tag(repo_path, &new_id, "tag")?;
+		tag_commit(repo_path, &new_id, "tag", None)?;
 
 		assert_eq!(
 			get_tags(repo_path).unwrap()[&new_id],
-			vec!["tag"]
+			vec![Tag::new("tag")]
 		);
 
-		assert!(matches!(tag(repo_path, &new_id, "tag"), Err(_)));
+		assert!(matches!(
+			tag_commit(repo_path, &new_id, "tag", None),
+			Err(_)
+		));
 
 		assert_eq!(
 			get_tags(repo_path).unwrap()[&new_id],
-			vec!["tag"]
+			vec![Tag::new("tag")]
 		);
 
-		tag(repo_path, &new_id, "second-tag")?;
+		tag_commit(repo_path, &new_id, "second-tag", None)?;
 
 		assert_eq!(
 			get_tags(repo_path).unwrap()[&new_id],
-			vec!["second-tag", "tag"]
+			vec![Tag::new("second-tag"), Tag::new("tag")]
+		);
+
+		Ok(())
+	}
+
+	#[test]
+	fn test_tag_with_message() -> Result<()> {
+		let file_path = Path::new("foo");
+		let (_td, repo) = repo_init_empty().unwrap();
+		let root = repo.path().parent().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
+
+		File::create(&root.join(file_path))?
+			.write_all(b"test\nfoo")?;
+
+		stage_add_file(repo_path, file_path)?;
+
+		let new_id = commit(repo_path, "commit msg")?;
+
+		tag_commit(repo_path, &new_id, "tag", Some("tag-message"))?;
+
+		assert_eq!(
+			get_tags(repo_path).unwrap()[&new_id][0]
+				.annotation
+				.as_ref()
+				.unwrap(),
+			"tag-message"
 		);
 
 		Ok(())
@@ -265,7 +312,8 @@ mod tests {
 		let file_path = Path::new("foo");
 		let (_td, repo) = repo_init_empty().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		File::create(&root.join(file_path))?
 			.write_all(b"test\nfoo")?;
@@ -300,7 +348,8 @@ mod tests {
 		let file_path = Path::new("foo");
 		let (_td, repo) = repo_init_empty().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		File::create(&root.join(file_path))?
 			.write_all(b"test\nfoo")?;
