@@ -1,11 +1,14 @@
-use std::cmp::Ordering;
+//! Functions for getting infos about files in commits
 
-use super::{stash::is_stash_commit, CommitId, RepoPath};
+use super::{
+	diff::DiffOptions, stash::is_stash_commit, CommitId, RepoPath,
+};
 use crate::{
 	error::Result, sync::repository::repo, StatusItem, StatusItemType,
 };
-use git2::{Diff, DiffOptions, Repository};
+use git2::{Diff, Repository};
 use scopetime::scope_time;
+use std::cmp::Ordering;
 
 /// get all files that are part of a commit
 pub fn get_commit_files(
@@ -18,9 +21,9 @@ pub fn get_commit_files(
 	let repo = repo(repo_path)?;
 
 	let diff = if let Some(other) = other {
-		get_compare_commits_diff(&repo, (id, other), None)?
+		get_compare_commits_diff(&repo, (id, other), None, None)?
 	} else {
-		get_commit_diff(repo_path, &repo, id, None)?
+		get_commit_diff(repo_path, &repo, id, None, None)?
 	};
 
 	let res = diff
@@ -42,11 +45,13 @@ pub fn get_commit_files(
 	Ok(res)
 }
 
+/// get diff of two arbitrary commits
 #[allow(clippy::needless_pass_by_value)]
 pub fn get_compare_commits_diff(
 	repo: &Repository,
 	ids: (CommitId, CommitId),
 	pathspec: Option<String>,
+	options: Option<DiffOptions>,
 ) -> Result<Diff<'_>> {
 	// scope_time!("get_compare_commits_diff");
 
@@ -65,7 +70,12 @@ pub fn get_compare_commits_diff(
 
 	let trees = (commits.0.tree()?, commits.1.tree()?);
 
-	let mut opts = DiffOptions::new();
+	let mut opts = git2::DiffOptions::new();
+	if let Some(options) = options {
+		opts.context_lines(options.context);
+		opts.ignore_whitespace(options.ignore_whitespace);
+		opts.interhunk_lines(options.interhunk_lines);
+	}
 	if let Some(p) = &pathspec {
 		opts.pathspec(p.clone());
 	}
@@ -80,12 +90,13 @@ pub fn get_compare_commits_diff(
 	Ok(diff)
 }
 
-#[allow(clippy::redundant_pub_crate)]
-pub(crate) fn get_commit_diff<'a>(
+/// get diff of a commit to its first parent
+pub fn get_commit_diff<'a>(
 	repo_path: &RepoPath,
 	repo: &'a Repository,
 	id: CommitId,
 	pathspec: Option<String>,
+	options: Option<DiffOptions>,
 ) -> Result<Diff<'a>> {
 	// scope_time!("get_commit_diff");
 
@@ -100,7 +111,12 @@ pub(crate) fn get_commit_diff<'a>(
 		None
 	};
 
-	let mut opts = DiffOptions::new();
+	let mut opts = git2::DiffOptions::new();
+	if let Some(options) = options {
+		opts.context_lines(options.context);
+		opts.ignore_whitespace(options.ignore_whitespace);
+		opts.interhunk_lines(options.interhunk_lines);
+	}
 	if let Some(p) = &pathspec {
 		opts.pathspec(p.clone());
 	}
@@ -119,6 +135,7 @@ pub(crate) fn get_commit_diff<'a>(
 				repo,
 				CommitId::new(untracked_commit),
 				pathspec,
+				options,
 			)?;
 
 			diff.merge(&untracked_diff)?;
