@@ -13,10 +13,11 @@ use git2::{
 	Delta, Diff, DiffDelta, DiffFormat, DiffHunk, Patch, Repository,
 };
 use scopetime::scope_time;
+use serde::{Deserialize, Serialize};
 use std::{cell::RefCell, fs, path::Path, rc::Rc};
 
 /// type of diff of a single line
-#[derive(Copy, Clone, PartialEq, Hash, Debug)]
+#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
 pub enum DiffLineType {
 	/// just surrounding line, no change
 	None,
@@ -127,7 +128,9 @@ pub struct FileDiff {
 }
 
 /// see <https://libgit2.org/libgit2/#HEAD/type/git_diff_options>
-#[derive(Debug, Hash, Clone, Copy, PartialEq)]
+#[derive(
+	Debug, Hash, Clone, Copy, PartialEq, Eq, Serialize, Deserialize,
+)]
 pub struct DiffOptions {
 	/// see <https://libgit2.org/libgit2/#HEAD/type/git_diff_options>
 	pub ignore_whitespace: bool,
@@ -214,12 +217,14 @@ pub fn get_diff_commit(
 	repo_path: &RepoPath,
 	id: CommitId,
 	p: String,
+	options: Option<DiffOptions>,
 ) -> Result<FileDiff> {
 	scope_time!("get_diff_commit");
 
 	let repo = repo(repo_path)?;
 	let work_dir = work_dir(&repo)?;
-	let diff = get_commit_diff(repo_path, &repo, id, Some(p))?;
+	let diff =
+		get_commit_diff(repo_path, &repo, id, Some(p), options)?;
 
 	raw_diff_to_file_diff(&diff, work_dir)
 }
@@ -229,13 +234,18 @@ pub fn get_diff_commits(
 	repo_path: &RepoPath,
 	ids: (CommitId, CommitId),
 	p: String,
+	options: Option<DiffOptions>,
 ) -> Result<FileDiff> {
 	scope_time!("get_diff_commits");
 
 	let repo = repo(repo_path)?;
 	let work_dir = work_dir(&repo)?;
-	let diff =
-		get_compare_commits_diff(&repo, (ids.0, ids.1), Some(p))?;
+	let diff = get_compare_commits_diff(
+		&repo,
+		(ids.0, ids.1),
+		Some(p),
+		options,
+	)?;
 
 	raw_diff_to_file_diff(&diff, work_dir)
 }
@@ -649,7 +659,8 @@ mod tests {
 		let id = commit(repo_path, "").unwrap();
 
 		let diff =
-			get_diff_commit(repo_path, id, String::new()).unwrap();
+			get_diff_commit(repo_path, id, String::new(), None)
+				.unwrap();
 
 		dbg!(&diff);
 		assert_eq!(diff.sizes, (1, 2));
