@@ -7,24 +7,29 @@ use crate::{
 		CommitComponent, CompareCommitsComponent, Component,
 		ConfirmComponent, CreateBranchComponent, DrawableComponent,
 		ExternalEditorComponent, FetchComponent, FileFindPopup,
-		HelpComponent, InspectCommitComponent, MsgComponent,
-		OptionsPopupComponent, PullComponent, PushComponent,
-		PushTagsComponent, RenameBranchComponent, RevisionFilesPopup,
-		SharedOptions, StashMsgComponent, TagCommitComponent,
+		FileRevlogComponent, HelpComponent, InspectCommitComponent,
+		MsgComponent, OptionsPopupComponent, PullComponent,
+		PushComponent, PushTagsComponent, RenameBranchComponent,
+		RevisionFilesPopup, StashMsgComponent,
+		SubmodulesListComponent, TagCommitComponent,
 		TagListComponent,
 	},
 	input::{Input, InputEvent, InputState},
-	keys::{KeyConfig, SharedKeyConfig},
-	queue::{Action, InternalEvent, NeedsUpdate, Queue},
+	keys::{key_match, KeyConfig, SharedKeyConfig},
+	options::{Options, SharedOptions},
+	popup_stack::PopupStack,
+	queue::{
+		Action, InternalEvent, NeedsUpdate, Queue, StackablePopupOpen,
+	},
 	setup_popups,
-	strings::{self, order},
+	strings::{self, ellipsis_trim_start, order},
 	tabs::{FilesTab, Revlog, StashList, Stashing, Status},
 	ui::style::{SharedTheme, Theme},
 	AsyncAppNotification, AsyncNotification,
 };
 use anyhow::{bail, Result};
 use asyncgit::{
-	sync::{self, RepoPathRef},
+	sync::{self, utils::repo_work_dir, RepoPath, RepoPathRef},
 	AsyncGitNotification, PushType,
 };
 use crossbeam_channel::Sender;
@@ -36,21 +41,32 @@ use std::{
 };
 use tui::{
 	backend::Backend,
-	layout::{Constraint, Direction, Layout, Margin, Rect},
+	layout::{
+		Alignment, Constraint, Direction, Layout, Margin, Rect,
+	},
 	text::{Span, Spans},
-	widgets::{Block, Borders, Tabs},
+	widgets::{Block, Borders, Paragraph, Tabs},
 	Frame,
 };
+use unicode_width::UnicodeWidthStr;
+
+#[derive(Clone)]
+pub enum QuitState {
+	None,
+	Close,
+	OpenSubmodule(RepoPath),
+}
 
 /// the main app type
 pub struct App {
 	repo: RepoPathRef,
-	do_quit: bool,
+	do_quit: QuitState,
 	help: HelpComponent,
 	msg: MsgComponent,
 	reset: ConfirmComponent,
 	commit: CommitComponent,
 	blame_file_popup: BlameFileComponent,
+	file_revlog_popup: FileRevlogComponent,
 	stashmsg_popup: StashMsgComponent,
 	inspect_commit_popup: InspectCommitComponent,
 	compare_commits_popup: CompareCommitsComponent,
@@ -66,6 +82,7 @@ pub struct App {
 	rename_branch_popup: RenameBranchComponent,
 	select_branch_popup: BranchListComponent,
 	options_popup: OptionsPopupComponent,
+	submodule_popup: SubmodulesListComponent,
 	tags_popup: TagListComponent,
 	cmdbar: RefCell<CommandBar>,
 	tab: usize,
@@ -78,6 +95,9 @@ pub struct App {
 	theme: SharedTheme,
 	key_config: SharedKeyConfig,
 	input: Input,
+	popup_stack: PopupStack,
+	options: SharedOptions,
+	repo_path_text: String,
 
 	// "Flags"
 	requires_redraw: Cell<bool>,
@@ -95,13 +115,20 @@ impl App {
 		input: Input,
 		theme: Theme,
 		key_config: KeyConfig,
-	) -> Self {
+	) -> Result<Self> {
+		log::trace!("open repo at: {:?}", &repo);
+
+		let repo_path_text =
+			repo_work_dir(&repo.borrow()).unwrap_or_default();
+
 		let queue = Queue::new();
 		let theme = Rc::new(theme);
 		let key_config = Rc::new(key_config);
-		let options = SharedOptions::default();
+		let options = Options::new(repo.clone());
 
-		Self {
+		let tab = options.borrow().current_tab();
+
+		let mut app = Self {
 			input,
 			reset: ConfirmComponent::new(
 				queue.clone(),
@@ -113,6 +140,7 @@ impl App {
 				queue.clone(),
 				theme.clone(),
 				key_config.clone(),
+				options.clone(),
 			),
 			blame_file_popup: BlameFileComponent::new(
 				&repo,
@@ -122,10 +150,19 @@ impl App {
 				theme.clone(),
 				key_config.clone(),
 			),
+			file_revlog_popup: FileRevlogComponent::new(
+				&repo,
+				&queue,
+				sender,
+				theme.clone(),
+				key_config.clone(),
+				options.clone(),
+			),
 			revision_files_popup: RevisionFilesPopup::new(
 				repo.clone(),
 				&queue,
 				sender_app,
+				sender.clone(),
 				theme.clone(),
 				key_config.clone(),
 			),
@@ -218,12 +255,18 @@ impl App {
 				key_config.clone(),
 				options.clone(),
 			),
+			submodule_popup: SubmodulesListComponent::new(
+				repo.clone(),
+				&queue,
+				theme.clone(),
+				key_config.clone(),
+			),
 			find_file_popup: FileFindPopup::new(
 				&queue,
 				theme.clone(),
 				key_config.clone(),
 			),
-			do_quit: false,
+			do_quit: QuitState::None,
 			cmdbar: RefCell::new(CommandBar::new(
 				theme.clone(),
 				key_config.clone(),
@@ -233,7 +276,6 @@ impl App {
 				key_config.clone(),
 			),
 			msg: MsgComponent::new(theme.clone(), key_config.clone()),
-			tab: 0,
 			revlog: Revlog::new(
 				&repo,
 				&queue,
@@ -247,7 +289,7 @@ impl App {
 				sender,
 				theme.clone(),
 				key_config.clone(),
-				options,
+				options.clone(),
 			),
 			stashing_tab: Stashing::new(
 				&repo,
@@ -265,17 +307,26 @@ impl App {
 			files_tab: FilesTab::new(
 				repo.clone(),
 				sender_app,
+				sender.clone(),
 				&queue,
 				theme.clone(),
 				key_config.clone(),
 			),
+			tab: 0,
 			queue,
 			theme,
+			options,
 			key_config,
 			requires_redraw: Cell::new(false),
 			file_to_open: None,
 			repo,
-		}
+			repo_path_text,
+			popup_stack: PopupStack::default(),
+		};
+
+		app.set_tab(tab)?;
+
+		Ok(app)
 	}
 
 	///
@@ -298,17 +349,28 @@ impl App {
 
 		self.cmdbar.borrow().draw(f, chunks_main[2]);
 
-		self.draw_tabs(f, chunks_main[0]);
+		self.draw_top_bar(f, chunks_main[0]);
 
-		//TODO: macro because of generic draw call
-		match self.tab {
-			0 => self.status_tab.draw(f, chunks_main[1])?,
-			1 => self.revlog.draw(f, chunks_main[1])?,
-			2 => self.files_tab.draw(f, chunks_main[1])?,
-			3 => self.stashing_tab.draw(f, chunks_main[1])?,
-			4 => self.stashlist_tab.draw(f, chunks_main[1])?,
-			_ => bail!("unknown tab"),
-		};
+		//TODO: component property + a macro `fullscreen_popup_open!`
+		// to make this scale better?
+		let fullscreen_popup_open =
+			self.revision_files_popup.is_visible()
+				|| self.inspect_commit_popup.is_visible()
+				|| self.compare_commits_popup.is_visible()
+				|| self.blame_file_popup.is_visible()
+				|| self.file_revlog_popup.is_visible();
+
+		if !fullscreen_popup_open {
+			//TODO: macro because of generic draw call
+			match self.tab {
+				0 => self.status_tab.draw(f, chunks_main[1])?,
+				1 => self.revlog.draw(f, chunks_main[1])?,
+				2 => self.files_tab.draw(f, chunks_main[1])?,
+				3 => self.stashing_tab.draw(f, chunks_main[1])?,
+				4 => self.stashlist_tab.draw(f, chunks_main[1])?,
+				_ => bail!("unknown tab"),
+			};
+		}
 
 		self.draw_popups(f)?;
 
@@ -320,38 +382,57 @@ impl App {
 		log::trace!("event: {:?}", ev);
 
 		if let InputEvent::Input(ev) = ev {
-			if self.check_hard_exit(ev) || self.check_quit(ev) {
+			if self.check_hard_exit(&ev) || self.check_quit(&ev) {
 				return Ok(());
 			}
 
 			let mut flags = NeedsUpdate::empty();
 
-			if event_pump(ev, self.components_mut().as_mut_slice())?
+			if event_pump(&ev, self.components_mut().as_mut_slice())?
 				.is_consumed()
 			{
 				flags.insert(NeedsUpdate::COMMANDS);
-			} else if let Event::Key(k) = ev {
-				let new_flags = if k
-					== self.key_config.keys.tab_toggle
-				{
+			} else if let Event::Key(k) = &ev {
+				let new_flags = if key_match(
+					k,
+					self.key_config.keys.tab_toggle,
+				) {
 					self.toggle_tabs(false)?;
 					NeedsUpdate::COMMANDS
-				} else if k == self.key_config.keys.tab_toggle_reverse
-				{
+				} else if key_match(
+					k,
+					self.key_config.keys.tab_toggle_reverse,
+				) {
 					self.toggle_tabs(true)?;
 					NeedsUpdate::COMMANDS
-				} else if k == self.key_config.keys.tab_status
-					|| k == self.key_config.keys.tab_log
-					|| k == self.key_config.keys.tab_files
-					|| k == self.key_config.keys.tab_stashing
-					|| k == self.key_config.keys.tab_stashes
-				{
+				} else if key_match(
+					k,
+					self.key_config.keys.tab_status,
+				) || key_match(
+					k,
+					self.key_config.keys.tab_log,
+				) || key_match(
+					k,
+					self.key_config.keys.tab_files,
+				) || key_match(
+					k,
+					self.key_config.keys.tab_stashing,
+				) || key_match(
+					k,
+					self.key_config.keys.tab_stashes,
+				) {
 					self.switch_tab(k)?;
 					NeedsUpdate::COMMANDS
-				} else if k == self.key_config.keys.cmd_bar_toggle {
+				} else if key_match(
+					k,
+					self.key_config.keys.cmd_bar_toggle,
+				) {
 					self.cmdbar.borrow_mut().toggle_more();
 					NeedsUpdate::empty()
-				} else if k == self.key_config.keys.open_options {
+				} else if key_match(
+					k,
+					self.key_config.keys.open_options,
+				) {
 					self.options_popup.show()?;
 					NeedsUpdate::ALL
 				} else {
@@ -377,7 +458,7 @@ impl App {
 
 				if let Err(e) = result {
 					let msg =
-						format!("failed to launch editor:\n{}", e);
+						format!("failed to launch editor:\n{e}");
 					log::error!("{}", msg.as_str());
 					self.msg.show_error(msg.as_str())?;
 				}
@@ -419,6 +500,7 @@ impl App {
 			self.stashing_tab.update_git(ev)?;
 			self.revlog.update_git(ev)?;
 			self.blame_file_popup.update_git(ev)?;
+			self.file_revlog_popup.update_git(ev)?;
 			self.inspect_commit_popup.update_git(ev)?;
 			self.compare_commits_popup.update_git(ev)?;
 			self.push_popup.update_git(ev)?;
@@ -428,8 +510,8 @@ impl App {
 			self.select_branch_popup.update_git(ev)?;
 		}
 
-		self.files_tab.update_async(ev);
-		self.revision_files_popup.update(ev);
+		self.files_tab.update_async(ev)?;
+		self.revision_files_popup.update(ev)?;
 		self.tags_popup.update(ev);
 
 		//TODO: better system for this
@@ -441,7 +523,13 @@ impl App {
 
 	///
 	pub fn is_quit(&self) -> bool {
-		self.do_quit || self.input.is_aborted()
+		!matches!(self.do_quit, QuitState::None)
+			|| self.input.is_aborted()
+	}
+
+	///
+	pub fn quit_state(&self) -> QuitState {
+		self.do_quit.clone()
 	}
 
 	///
@@ -451,6 +539,7 @@ impl App {
 			|| self.stashing_tab.anything_pending()
 			|| self.files_tab.anything_pending()
 			|| self.blame_file_popup.any_work_pending()
+			|| self.file_revlog_popup.any_work_pending()
 			|| self.inspect_commit_popup.any_work_pending()
 			|| self.compare_commits_popup.any_work_pending()
 			|| self.input.is_state_changing()
@@ -483,6 +572,7 @@ impl App {
 			reset,
 			commit,
 			blame_file_popup,
+			file_revlog_popup,
 			stashmsg_popup,
 			inspect_commit_popup,
 			compare_commits_popup,
@@ -496,6 +586,7 @@ impl App {
 			rename_branch_popup,
 			select_branch_popup,
 			revision_files_popup,
+			submodule_popup,
 			tags_popup,
 			options_popup,
 			help,
@@ -516,9 +607,11 @@ impl App {
 			inspect_commit_popup,
 			compare_commits_popup,
 			blame_file_popup,
+			file_revlog_popup,
 			external_editor_popup,
 			tag_commit_popup,
 			select_branch_popup,
+			submodule_popup,
 			tags_popup,
 			create_branch_popup,
 			rename_branch_popup,
@@ -534,23 +627,23 @@ impl App {
 		]
 	);
 
-	fn check_quit(&mut self, ev: Event) -> bool {
+	fn check_quit(&mut self, ev: &Event) -> bool {
 		if self.any_popup_visible() {
 			return false;
 		}
 		if let Event::Key(e) = ev {
-			if e == self.key_config.keys.quit {
-				self.do_quit = true;
+			if key_match(e, self.key_config.keys.quit) {
+				self.do_quit = QuitState::Close;
 				return true;
 			}
 		}
 		false
 	}
 
-	fn check_hard_exit(&mut self, ev: Event) -> bool {
+	fn check_hard_exit(&mut self, ev: &Event) -> bool {
 		if let Event::Key(e) = ev {
-			if e == self.key_config.keys.exit {
-				self.do_quit = true;
+			if key_match(e, self.key_config.keys.exit) {
+				self.do_quit = QuitState::Close;
 				return true;
 			}
 		}
@@ -578,16 +671,16 @@ impl App {
 		self.set_tab(new_tab)
 	}
 
-	fn switch_tab(&mut self, k: KeyEvent) -> Result<()> {
-		if k == self.key_config.keys.tab_status {
+	fn switch_tab(&mut self, k: &KeyEvent) -> Result<()> {
+		if key_match(k, self.key_config.keys.tab_status) {
 			self.set_tab(0)?;
-		} else if k == self.key_config.keys.tab_log {
+		} else if key_match(k, self.key_config.keys.tab_log) {
 			self.set_tab(1)?;
-		} else if k == self.key_config.keys.tab_files {
+		} else if key_match(k, self.key_config.keys.tab_files) {
 			self.set_tab(2)?;
-		} else if k == self.key_config.keys.tab_stashing {
+		} else if key_match(k, self.key_config.keys.tab_stashing) {
 			self.set_tab(3)?;
-		} else if k == self.key_config.keys.tab_stashes {
+		} else if key_match(k, self.key_config.keys.tab_stashes) {
 			self.set_tab(4)?;
 		}
 
@@ -605,6 +698,7 @@ impl App {
 		}
 
 		self.tab = tab;
+		self.options.borrow_mut().set_current_tab(tab);
 
 		Ok(())
 	}
@@ -630,6 +724,7 @@ impl App {
 			self.status_tab.update_diff()?;
 			self.inspect_commit_popup.update_diff()?;
 			self.compare_commits_popup.update_diff()?;
+			self.file_revlog_popup.update_diff()?;
 		}
 		if flags.contains(NeedsUpdate::COMMANDS) {
 			self.update_commands();
@@ -641,6 +736,31 @@ impl App {
 		Ok(())
 	}
 
+	fn open_popup(
+		&mut self,
+		popup: StackablePopupOpen,
+	) -> Result<()> {
+		match popup {
+			StackablePopupOpen::BlameFile(params) => {
+				self.blame_file_popup.open(params)?;
+			}
+			StackablePopupOpen::FileRevlog(param) => {
+				self.file_revlog_popup.open(param)?;
+			}
+			StackablePopupOpen::FileTree(param) => {
+				self.revision_files_popup.open(param)?;
+			}
+			StackablePopupOpen::InspectCommit(param) => {
+				self.inspect_commit_popup.open(param)?;
+			}
+			StackablePopupOpen::CompareCommits(param) => {
+				self.compare_commits_popup.open(param)?;
+			}
+		}
+
+		Ok(())
+	}
+
 	fn process_internal_events(&mut self) -> Result<NeedsUpdate> {
 		let mut flags = NeedsUpdate::empty();
 
@@ -690,11 +810,7 @@ impl App {
 			InternalEvent::TagCommit(id) => {
 				self.tag_commit_popup.open(id)?;
 			}
-			InternalEvent::BlameFile(path) => {
-				self.blame_file_popup.open(&path)?;
-				flags
-					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
-			}
+
 			InternalEvent::CreateBranch => {
 				self.create_branch_popup.open()?;
 			}
@@ -705,15 +821,13 @@ impl App {
 			InternalEvent::SelectBranch => {
 				self.select_branch_popup.open()?;
 			}
+			InternalEvent::ViewSubmodules => {
+				self.submodule_popup.open()?;
+			}
 			InternalEvent::Tags => {
 				self.tags_popup.open()?;
 			}
 			InternalEvent::TabSwitchStatus => self.set_tab(0)?,
-			InternalEvent::InspectCommit(id, tags) => {
-				self.inspect_commit_popup.open(id, tags)?;
-				flags
-					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
-			}
 			InternalEvent::SelectCommitInRevlog(id) => {
 				if let Err(error) = self.revlog.select_commit(id) {
 					self.queue.push(InternalEvent::ShowErrorMsg(
@@ -758,11 +872,6 @@ impl App {
 			InternalEvent::StatusLastFileMoved => {
 				self.status_tab.last_file_moved()?;
 			}
-			InternalEvent::OpenFileTree(c) => {
-				self.revision_files_popup.open(c)?;
-				flags
-					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
-			}
 			InternalEvent::OpenFileFinder(files) => {
 				self.find_file_popup.open(&files)?;
 				flags
@@ -782,17 +891,39 @@ impl App {
 
 				flags.insert(NeedsUpdate::ALL);
 			}
-			InternalEvent::CompareCommits(id, other) => {
-				self.compare_commits_popup.open(id, other)?;
-				flags
-					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
-			}
 			InternalEvent::FileFinderChanged(file) => {
 				self.files_tab.file_finder_update(&file);
 				self.revision_files_popup.file_finder_update(&file);
 				flags
 					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
 			}
+			InternalEvent::OpenPopup(popup) => {
+				self.open_popup(popup)?;
+				flags
+					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
+			}
+			InternalEvent::PopupStackPop => {
+				if let Some(popup) = self.popup_stack.pop() {
+					self.open_popup(popup)?;
+					flags.insert(
+						NeedsUpdate::ALL | NeedsUpdate::COMMANDS,
+					);
+				}
+			}
+			InternalEvent::PopupStackPush(popup) => {
+				self.popup_stack.push(popup);
+				flags
+					.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
+			}
+			InternalEvent::OpenRepo { path } => {
+				let submodule_repo_path = RepoPath::Path(
+					Path::new(&repo_work_dir(&self.repo.borrow())?)
+						.join(path),
+				);
+				//TODO: validate this is a valid repo first, so we can show proper error otherwise
+				self.do_quit =
+					QuitState::OpenSubmodule(submodule_repo_path);
+			}
 		};
 
 		Ok(flags)
@@ -811,10 +942,10 @@ impl App {
 				}
 			}
 			Action::StashDrop(_) | Action::StashPop(_) => {
-				if let Err(e) = StashList::action_confirmed(
-					&self.repo.borrow(),
-					&action,
-				) {
+				if let Err(e) = self
+					.stashlist_tab
+					.action_confirmed(&self.repo.borrow(), &action)
+				{
 					self.queue.push(InternalEvent::ShowErrorMsg(
 						e.to_string(),
 					));
@@ -983,23 +1114,47 @@ impl App {
 	}
 
 	//TODO: make this dynamic
-	fn draw_tabs<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
+	fn draw_top_bar<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
+		const DIVIDER_PAD_SPACES: usize = 2;
+		const SIDE_PADS: usize = 2;
+		const MARGIN_LEFT_AND_RIGHT: usize = 2;
+
 		let r = r.inner(&Margin {
 			vertical: 0,
 			horizontal: 1,
 		});
 
-		let tabs = [
+		let tab_labels = [
 			Span::raw(strings::tab_status(&self.key_config)),
 			Span::raw(strings::tab_log(&self.key_config)),
 			Span::raw(strings::tab_files(&self.key_config)),
 			Span::raw(strings::tab_stashing(&self.key_config)),
 			Span::raw(strings::tab_stashes(&self.key_config)),
-		]
-		.iter()
-		.cloned()
-		.map(Spans::from)
-		.collect();
+		];
+		let divider = strings::tab_divider(&self.key_config);
+
+		// heuristic, since tui doesn't provide a way to know
+		// how much space is needed to draw a `Tabs`
+		let tabs_len: usize =
+			tab_labels.iter().map(Span::width).sum::<usize>()
+				+ tab_labels.len().saturating_sub(1)
+					* (divider.width() + DIVIDER_PAD_SPACES)
+				+ SIDE_PADS + MARGIN_LEFT_AND_RIGHT;
+
+		let left_right = Layout::default()
+			.direction(Direction::Horizontal)
+			.constraints(vec![
+				Constraint::Length(
+					u16::try_from(tabs_len).unwrap_or(r.width),
+				),
+				Constraint::Min(0),
+			])
+			.split(r);
+
+		let table_area = r; // use entire area to allow drawing the horizontal separator line
+		let text_area = left_right[1];
+
+		let tabs = tab_labels.into_iter().map(Spans::from).collect();
 
 		f.render_widget(
 			Tabs::new(tabs)
@@ -1010,9 +1165,21 @@ impl App {
 				)
 				.style(self.theme.tab(false))
 				.highlight_style(self.theme.tab(true))
-				.divider(strings::tab_divider(&self.key_config))
+				.divider(divider)
 				.select(self.tab),
-			r,
+			table_area,
+		);
+
+		f.render_widget(
+			Paragraph::new(Spans::from(vec![Span::styled(
+				ellipsis_trim_start(
+					&self.repo_path_text,
+					text_area.width as usize,
+				),
+				self.theme.title(true),
+			)]))
+			.alignment(Alignment::Right),
+			text_area,
 		);
 	}
 }
