@@ -4,7 +4,8 @@ use super::{
 	EventState, ExternalEditorComponent,
 };
 use crate::{
-	keys::SharedKeyConfig,
+	keys::{key_match, SharedKeyConfig},
+	options::SharedOptions,
 	queue::{InternalEvent, NeedsUpdate, Queue},
 	strings, try_or_popup,
 	ui::style::SharedTheme,
@@ -51,6 +52,8 @@ pub struct CommitComponent {
 	git_branch_name: cached::BranchName,
 	commit_template: Option<String>,
 	theme: SharedTheme,
+	commit_msg_history_idx: usize,
+	options: SharedOptions,
 }
 
 const FIRST_LINE_LIMIT: usize = 50;
@@ -62,6 +65,7 @@ impl CommitComponent {
 		queue: Queue,
 		theme: SharedTheme,
 		key_config: SharedKeyConfig,
+		options: SharedOptions,
 	) -> Self {
 		Self {
 			queue,
@@ -78,6 +82,8 @@ impl CommitComponent {
 			commit_template: None,
 			theme,
 			repo,
+			commit_msg_history_idx: 0,
+			options,
 		}
 	}
 
@@ -88,7 +94,7 @@ impl CommitComponent {
 
 	fn draw_branch_name<B: Backend>(&self, f: &mut Frame<B>) {
 		if let Some(name) = self.git_branch_name.last() {
-			let w = Paragraph::new(format!("{{{}}}", name))
+			let w = Paragraph::new(format!("{{{name}}}"))
 				.alignment(Alignment::Right);
 
 			let rect = {
@@ -186,6 +192,11 @@ impl CommitComponent {
 			self.commit_with_msg(msg)?,
 			CommitResult::ComitDone
 		) {
+			self.options
+				.borrow_mut()
+				.add_commit_msg(self.input.get_text());
+			self.commit_msg_history_idx = 0;
+
 			self.hide();
 			self.queue.push(InternalEvent::Update(NeedsUpdate::ALL));
 			self.input.clear();
@@ -328,19 +339,27 @@ impl Component for CommitComponent {
 				true,
 				true,
 			));
+
+			out.push(CommandInfo::new(
+				strings::commands::commit_next_msg_from_history(
+					&self.key_config,
+				),
+				true,
+				true,
+			));
 		}
 
 		visibility_blocking(self)
 	}
 
-	fn event(&mut self, ev: Event) -> Result<EventState> {
+	fn event(&mut self, ev: &Event) -> Result<EventState> {
 		if self.is_visible() {
 			if self.input.event(ev)?.is_consumed() {
 				return Ok(EventState::Consumed);
 			}
 
 			if let Event::Key(e) = ev {
-				if e == self.key_config.keys.enter
+				if key_match(e, self.key_config.keys.enter)
 					&& self.can_commit()
 				{
 					try_or_popup!(
@@ -348,16 +367,32 @@ impl Component for CommitComponent {
 						"commit error:",
 						self.commit()
 					);
-				} else if e == self.key_config.keys.commit_amend
-					&& self.can_amend()
+				} else if key_match(
+					e,
+					self.key_config.keys.commit_amend,
+				) && self.can_amend()
 				{
 					self.amend()?;
-				} else if e == self.key_config.keys.open_commit_editor
-				{
+				} else if key_match(
+					e,
+					self.key_config.keys.open_commit_editor,
+				) {
 					self.queue.push(
 						InternalEvent::OpenExternalEditor(None),
 					);
 					self.hide();
+				} else if key_match(
+					e,
+					self.key_config.keys.commit_history_next,
+				) {
+					if let Some(msg) = self
+						.options
+						.borrow()
+						.commit_msg(self.commit_msg_history_idx)
+					{
+						self.input.set_text(msg);
+						self.commit_msg_history_idx += 1;
+					}
 				} else {
 				}
 				// stop key event propagation
@@ -420,6 +455,7 @@ impl Component for CommitComponent {
 			}
 		};
 
+		self.commit_msg_history_idx = 0;
 		self.input.show()?;
 
 		Ok(())
