@@ -1,10 +1,13 @@
 use super::{
 	command_pump, event_pump, visibility_blocking, CommandBlocking,
 	CommandInfo, CommitDetailsComponent, Component, DiffComponent,
-	DrawableComponent, EventState,
+	DrawableComponent, EventState, InspectCommitOpen,
 };
 use crate::{
-	accessors, keys::SharedKeyConfig, queue::Queue, strings,
+	accessors,
+	keys::{key_match, SharedKeyConfig},
+	queue::{InternalEvent, Queue, StackablePopupOpen},
+	strings,
 	ui::style::SharedTheme,
 };
 use anyhow::Result;
@@ -24,12 +27,13 @@ use tui::{
 
 pub struct CompareCommitsComponent {
 	repo: RepoPathRef,
-	commit_ids: Option<(CommitId, CommitId)>,
+	open_request: Option<InspectCommitOpen>,
 	diff: DiffComponent,
 	details: CommitDetailsComponent,
 	git_diff: AsyncDiff,
 	visible: bool,
 	key_config: SharedKeyConfig,
+	queue: Queue,
 }
 
 impl DrawableComponent for CompareCommitsComponent {
@@ -104,29 +108,39 @@ impl Component for CompareCommitsComponent {
 		visibility_blocking(self)
 	}
 
-	fn event(&mut self, ev: Event) -> Result<EventState> {
+	fn event(&mut self, ev: &Event) -> Result<EventState> {
 		if self.is_visible() {
 			if event_pump(ev, self.components_mut().as_mut_slice())?
 				.is_consumed()
 			{
+				if !self.details.is_visible() {
+					self.hide_stacked(true);
+				}
 				return Ok(EventState::Consumed);
 			}
 
 			if let Event::Key(e) = ev {
-				if e == self.key_config.keys.exit_popup {
-					self.hide();
-				} else if e == self.key_config.keys.focus_right
-					&& self.can_focus_diff()
+				if key_match(e, self.key_config.keys.exit_popup) {
+					self.hide_stacked(false);
+				} else if key_match(
+					e,
+					self.key_config.keys.focus_right,
+				) && self.can_focus_diff()
 				{
 					self.details.focus(false);
 					self.diff.focus(true);
-				} else if e == self.key_config.keys.focus_left
-					&& self.diff.focused()
+				} else if key_match(
+					e,
+					self.key_config.keys.focus_left,
+				) && self.diff.focused()
 				{
 					self.details.focus(true);
 					self.diff.focus(false);
-				} else if e == self.key_config.keys.focus_left {
-					self.hide();
+				} else if key_match(
+					e,
+					self.key_config.keys.focus_left,
+				) {
+					self.hide_stacked(false);
 				}
 
 				return Ok(EventState::Consumed);
@@ -179,25 +193,26 @@ impl CompareCommitsComponent {
 				key_config.clone(),
 				true,
 			),
-			commit_ids: None,
+			open_request: None,
 			git_diff: AsyncDiff::new(repo.borrow().clone(), sender),
 			visible: false,
 			key_config,
+			queue: queue.clone(),
 		}
 	}
 
 	///
-	pub fn open(
-		&mut self,
-		id: CommitId,
-		other: Option<CommitId>,
-	) -> Result<()> {
-		let other = if let Some(other) = other {
-			other
+	pub fn open(&mut self, open: InspectCommitOpen) -> Result<()> {
+		let compare_id = if let Some(compare_id) = open.compare_id {
+			compare_id
 		} else {
 			sync::get_head_tuple(&self.repo.borrow())?.id
 		};
-		self.commit_ids = Some((id, other));
+		self.open_request = Some(InspectCommitOpen {
+			commit_id: open.commit_id,
+			compare_id: Some(compare_id),
+			tags: open.tags,
+		});
 		self.show()?;
 
 		Ok(())
@@ -224,10 +239,22 @@ impl CompareCommitsComponent {
 		Ok(())
 	}
 
+	fn get_ids(&self) -> Option<(CommitId, CommitId)> {
+		let other = self
+			.open_request
+			.as_ref()
+			.and_then(|open| open.compare_id);
+
+		self.open_request
+			.as_ref()
+			.map(|open| open.commit_id)
+			.zip(other)
+	}
+
 	/// called when any tree component changed selection
 	pub fn update_diff(&mut self) -> Result<()> {
 		if self.is_visible() {
-			if let Some(ids) = self.commit_ids {
+			if let Some(ids) = self.get_ids() {
 				if let Some(f) = self.details.files().selection_file()
 				{
 					let diff_params = DiffParams {
@@ -259,8 +286,8 @@ impl CompareCommitsComponent {
 
 	fn update(&mut self) -> Result<()> {
 		self.details.set_commits(
-			self.commit_ids.map(CommitFilesParams::from),
-			None,
+			self.get_ids().map(CommitFilesParams::from),
+			&None,
 		)?;
 		self.update_diff()?;
 
@@ -270,4 +297,17 @@ impl CompareCommitsComponent {
 	fn can_focus_diff(&self) -> bool {
 		self.details.files().selection_file().is_some()
 	}
+
+	fn hide_stacked(&mut self, stack: bool) {
+		self.hide();
+		if stack {
+			if let Some(request) = self.open_request.clone() {
+				self.queue.push(InternalEvent::PopupStackPush(
+					StackablePopupOpen::CompareCommits(request),
+				));
+			}
+		} else {
+			self.queue.push(InternalEvent::PopupStackPop);
+		}
+	}
 }
