@@ -4,7 +4,7 @@ use super::{
 	EventState,
 };
 use crate::{
-	keys::SharedKeyConfig,
+	keys::{key_match, SharedKeyConfig},
 	queue::{InternalEvent, NeedsUpdate, Queue},
 	strings,
 	ui::style::SharedTheme,
@@ -61,14 +61,14 @@ impl Component for CreateBranchComponent {
 		visibility_blocking(self)
 	}
 
-	fn event(&mut self, ev: Event) -> Result<EventState> {
+	fn event(&mut self, ev: &Event) -> Result<EventState> {
 		if self.is_visible() {
 			if self.input.event(ev)?.is_consumed() {
 				return Ok(EventState::Consumed);
 			}
 
 			if let Event::Key(e) = ev {
-				if e == self.key_config.keys.enter {
+				if key_match(e, self.key_config.keys.enter) {
 					self.create_branch();
 				}
 
@@ -142,7 +142,7 @@ impl CreateBranchComponent {
 			Err(e) => {
 				log::error!("create branch: {}", e,);
 				self.queue.push(InternalEvent::ShowErrorMsg(
-					format!("create branch error:\n{}", e,),
+					format!("create branch error:\n{e}",),
 				));
 			}
 		}
