@@ -3,7 +3,7 @@ use crate::{
 		cred::CredComponent, visibility_blocking, CommandBlocking,
 		CommandInfo, Component, DrawableComponent, EventState,
 	},
-	keys::SharedKeyConfig,
+	keys::{key_match, SharedKeyConfig},
 	queue::{InternalEvent, Queue},
 	strings::{self},
 	ui::{self, style::SharedTheme},
@@ -123,7 +123,7 @@ impl PushTagsComponent {
 		if !self.pending {
 			if let Some(err) = self.git_push.last_result()? {
 				self.queue.push(InternalEvent::ShowErrorMsg(
-					format!("push tags failed:\n{}", err),
+					format!("push tags failed:\n{err}"),
 				));
 			}
 			self.hide();
@@ -227,7 +227,7 @@ impl Component for PushTagsComponent {
 		visibility_blocking(self)
 	}
 
-	fn event(&mut self, ev: Event) -> Result<EventState> {
+	fn event(&mut self, ev: &Event) -> Result<EventState> {
 		if self.visible {
 			if let Event::Key(e) = ev {
 				if self.input_cred.is_visible() {
@@ -241,8 +241,10 @@ impl Component for PushTagsComponent {
 						))?;
 						self.input_cred.hide();
 					}
-				} else if e == self.key_config.keys.exit_popup
-					&& !self.pending
+				} else if key_match(
+					e,
+					self.key_config.keys.exit_popup,
+				) && !self.pending
 				{
 					self.hide();
 				}
