@@ -6,22 +6,42 @@ use super::{
 	EventState,
 };
 use crate::{
-	keys::SharedKeyConfig,
-	queue::Queue,
+	keys::{key_match, SharedKeyConfig},
+	queue::{InternalEvent, Queue, StackablePopupOpen},
 	strings::{self},
 	ui::style::SharedTheme,
 	AsyncAppNotification, AsyncNotification,
 };
 use anyhow::Result;
-use asyncgit::sync::{CommitId, RepoPathRef};
+use asyncgit::{
+	sync::{CommitId, RepoPathRef},
+	AsyncGitNotification,
+};
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
 use tui::{backend::Backend, layout::Rect, widgets::Clear, Frame};
 
+#[derive(Clone, Debug)]
+pub struct FileTreeOpen {
+	pub commit_id: CommitId,
+	pub selection: Option<usize>,
+}
+
+impl FileTreeOpen {
+	pub const fn new(commit_id: CommitId) -> Self {
+		Self {
+			commit_id,
+			selection: None,
+		}
+	}
+}
+
 pub struct RevisionFilesPopup {
+	open_request: Option<FileTreeOpen>,
 	visible: bool,
 	key_config: SharedKeyConfig,
 	files: RevisionFilesComponent,
+	queue: Queue,
 }
 
 impl RevisionFilesPopup {
@@ -30,6 +50,7 @@ impl RevisionFilesPopup {
 		repo: RepoPathRef,
 		queue: &Queue,
 		sender: &Sender<AsyncAppNotification>,
+		sender_git: Sender<AsyncGitNotification>,
 		theme: SharedTheme,
 		key_config: SharedKeyConfig,
 	) -> Self {
@@ -38,25 +59,29 @@ impl RevisionFilesPopup {
 				repo,
 				queue,
 				sender,
+				sender_git,
 				theme,
 				key_config.clone(),
 			),
 			visible: false,
 			key_config,
+			open_request: None,
+			queue: queue.clone(),
 		}
 	}
 
 	///
-	pub fn open(&mut self, commit: CommitId) -> Result<()> {
-		self.files.set_commit(commit)?;
+	pub fn open(&mut self, request: FileTreeOpen) -> Result<()> {
+		self.files.set_commit(request.commit_id)?;
+		self.open_request = Some(request);
 		self.show()?;
 
 		Ok(())
 	}
 
 	///
-	pub fn update(&mut self, ev: AsyncNotification) {
-		self.files.update(ev);
+	pub fn update(&mut self, ev: AsyncNotification) -> Result<()> {
+		self.files.update(ev)
 	}
 
 	///
@@ -67,6 +92,23 @@ impl RevisionFilesPopup {
 	pub fn file_finder_update(&mut self, file: &Option<PathBuf>) {
 		self.files.find_file(file);
 	}
+
+	fn hide_stacked(&mut self, stack: bool) {
+		self.hide();
+
+		if stack {
+			if let Some(revision) = self.files.revision() {
+				self.queue.push(InternalEvent::PopupStackPush(
+					StackablePopupOpen::FileTree(FileTreeOpen {
+						commit_id: revision.id,
+						selection: self.files.selection(),
+					}),
+				));
+			}
+		} else {
+			self.queue.push(InternalEvent::PopupStackPop);
+		}
+	}
 }
 
 impl DrawableComponent for RevisionFilesPopup {
@@ -109,16 +151,23 @@ impl Component for RevisionFilesPopup {
 
 	fn event(
 		&mut self,
-		event: crossterm::event::Event,
+		event: &crossterm::event::Event,
 	) -> Result<EventState> {
 		if self.is_visible() {
-			if let Event::Key(key) = &event {
-				if *key == self.key_config.keys.exit_popup {
-					self.hide();
+			if let Event::Key(key) = event {
+				if key_match(key, self.key_config.keys.exit_popup) {
+					self.hide_stacked(false);
 				}
 			}
 
-			return self.files.event(event);
+			let res = self.files.event(event)?;
+			//Note: if this made the files hide we need to stack the popup
+			if res == EventState::Consumed && !self.files.is_visible()
+			{
+				self.hide_stacked(true);
+			}
+
+			return Ok(res);
 		}
 
 		Ok(EventState::NotConsumed)
