@@ -1,4 +1,8 @@
+use std::borrow::Cow;
+
 use asyncgit::sync::CommitId;
+use unicode_truncate::UnicodeTruncateStr;
+use unicode_width::UnicodeWidthStr;
 
 use crate::keys::SharedKeyConfig;
 
@@ -23,6 +27,12 @@ pub static PUSH_TAGS_STATES_FETCHING: &str = "fetching";
 pub static PUSH_TAGS_STATES_PUSHING: &str = "pushing";
 pub static PUSH_TAGS_STATES_DONE: &str = "done";
 
+pub static POPUP_TITLE_SUBMODULES: &str = "Submodules";
+pub static POPUP_TITLE_FUZZY_FIND: &str = "Fuzzy Finder";
+
+pub static POPUP_FAIL_COPY: &str = "Failed to copy text";
+pub static POPUP_SUCCESS_COPY: &str = "Copied Text";
+
 pub mod symbol {
 	pub const WHITESPACE: &str = "\u{00B7}"; //·
 	pub const CHECKMARK: &str = "\u{2713}"; //✓
@@ -31,6 +41,7 @@ pub mod symbol {
 	pub const FOLDER_ICON_COLLAPSED: &str = "\u{25b8}"; //▸
 	pub const FOLDER_ICON_EXPANDED: &str = "\u{25be}"; //▾
 	pub const EMPTY_STR: &str = "";
+	pub const ELLIPSIS: char = '\u{2026}'; // …
 }
 
 pub fn title_branches() -> String {
@@ -107,7 +118,7 @@ pub fn commit_msg(_key_config: &SharedKeyConfig) -> String {
 	"type commit message..".to_string()
 }
 pub fn commit_first_line_warning(count: usize) -> String {
-	format!("[subject length: {}]", count)
+	format!("[subject length: {count}]")
 }
 pub const fn branch_name_invalid() -> &'static str {
 	"[invalid name]"
@@ -154,9 +165,9 @@ pub fn confirm_msg_merge(
 	rebase: bool,
 ) -> String {
 	if rebase {
-		format!("Rebase onto {} incoming commits?", incoming)
+		format!("Rebase onto {incoming} incoming commits?")
 	} else {
-		format!("Merge of {} incoming commits?", incoming)
+		format!("Merge of {incoming} incoming commits?")
 	}
 }
 
@@ -222,7 +233,7 @@ pub fn confirm_msg_delete_branch(
 	_key_config: &SharedKeyConfig,
 	branch_ref: &str,
 ) -> String {
-	format!("Confirm deleting branch: '{}' ?", branch_ref)
+	format!("Confirm deleting branch: '{branch_ref}' ?")
 }
 pub fn confirm_title_delete_remote_branch(
 	_key_config: &SharedKeyConfig,
@@ -233,7 +244,7 @@ pub fn confirm_msg_delete_remote_branch(
 	_key_config: &SharedKeyConfig,
 	branch_ref: &str,
 ) -> String {
-	format!("Confirm deleting remote branch: '{}' ?", branch_ref)
+	format!("Confirm deleting remote branch: '{branch_ref}' ?")
 }
 pub fn confirm_title_delete_tag(
 	_key_config: &SharedKeyConfig,
@@ -244,13 +255,13 @@ pub fn confirm_msg_delete_tag(
 	_key_config: &SharedKeyConfig,
 	tag_name: &str,
 ) -> String {
-	format!("Confirm deleting Tag: '{}' ?", tag_name)
+	format!("Confirm deleting Tag: '{tag_name}' ?")
 }
 pub fn confirm_title_delete_tag_remote() -> String {
 	"Delete Tag (remote)".to_string()
 }
 pub fn confirm_msg_delete_tag_remote(remote_name: &str) -> String {
-	format!("Confirm deleting tag on remote '{}'?", remote_name)
+	format!("Confirm deleting tag on remote '{remote_name}'?")
 }
 pub fn confirm_title_force_push(
 	_key_config: &SharedKeyConfig,
@@ -269,6 +280,16 @@ pub fn confirm_msg_force_push(
 pub fn log_title(_key_config: &SharedKeyConfig) -> String {
 	"Commit".to_string()
 }
+pub fn file_log_title(
+	file_path: &str,
+	selected: usize,
+	revisions: usize,
+) -> String {
+	format!(
+		"Revisions of '{}' ({}/{})",
+		file_path, selected, revisions
+	)
+}
 pub fn blame_title(_key_config: &SharedKeyConfig) -> String {
 	"Blame".to_string()
 }
@@ -279,7 +300,7 @@ pub fn tag_popup_name_msg() -> String {
 	"type tag name".to_string()
 }
 pub fn tag_popup_annotation_title(name: &str) -> String {
-	format!("Tag Annotation ({})", name)
+	format!("Tag Annotation ({name})")
 }
 pub fn tag_popup_annotation_msg() -> String {
 	"type tag annotation".to_string()
@@ -335,6 +356,25 @@ pub fn rename_branch_popup_msg(
 	"new branch name".to_string()
 }
 
+pub fn copy_success(s: &str) -> String {
+	format!("{POPUP_SUCCESS_COPY} \"{s}\"")
+}
+
+pub fn ellipsis_trim_start(s: &str, width: usize) -> Cow<str> {
+	if s.width() <= width {
+		Cow::Borrowed(s)
+	} else {
+		Cow::Owned(format!(
+			"[{}]{}",
+			symbol::ELLIPSIS,
+			s.unicode_truncate_start(
+				width.saturating_sub(3 /* front indicator */)
+			)
+			.0
+		))
+	}
+}
+
 pub mod commit {
 	use crate::keys::SharedKeyConfig;
 
@@ -365,7 +405,7 @@ pub mod commit {
 		old: bool,
 		hash: &str,
 	) -> String {
-		format!("{}: {}", if old { "Old" } else { "New" }, hash)
+		format!("{}: {hash}", if old { "Old" } else { "New" })
 	}
 	pub fn details_message_title(
 		_key_config: &SharedKeyConfig,
@@ -690,6 +730,59 @@ pub mod commands {
 		)
 	}
 
+	pub fn view_submodules(
+		key_config: &SharedKeyConfig,
+	) -> CommandText {
+		CommandText::new(
+			format!(
+				"Submodules [{}]",
+				key_config.get_hint(key_config.keys.view_submodules),
+			),
+			"open submodule view",
+			CMD_GROUP_GENERAL,
+		)
+	}
+
+	pub fn open_submodule(
+		key_config: &SharedKeyConfig,
+	) -> CommandText {
+		CommandText::new(
+			format!(
+				"Open [{}]",
+				key_config.get_hint(key_config.keys.enter),
+			),
+			"open submodule",
+			CMD_GROUP_GENERAL,
+		)
+	}
+
+	pub fn open_submodule_parent(
+		key_config: &SharedKeyConfig,
+	) -> CommandText {
+		CommandText::new(
+			format!(
+				"Open Parent [{}]",
+				key_config
+					.get_hint(key_config.keys.view_submodule_parent),
+			),
+			"open submodule parent repo",
+			CMD_GROUP_GENERAL,
+		)
+	}
+
+	pub fn update_submodule(
+		key_config: &SharedKeyConfig,
+	) -> CommandText {
+		CommandText::new(
+			format!(
+				"Update [{}]",
+				key_config.get_hint(key_config.keys.update_submodule),
+			),
+			"update submodule",
+			CMD_GROUP_GENERAL,
+		)
+	}
+
 	pub fn continue_rebase(
 		key_config: &SharedKeyConfig,
 	) -> CommandText {
@@ -771,6 +864,19 @@ pub mod commands {
 			CMD_GROUP_COMMIT,
 		)
 	}
+	pub fn commit_next_msg_from_history(
+		key_config: &SharedKeyConfig,
+	) -> CommandText {
+		CommandText::new(
+			format!(
+				"Previous Msg [{}]",
+				key_config
+					.get_hint(key_config.keys.commit_history_next),
+			),
+			"use previous commit message from history",
+			CMD_GROUP_COMMIT,
+		)
+	}
 	pub fn commit_enter(key_config: &SharedKeyConfig) -> CommandText {
 		CommandText::new(
 			format!(
@@ -986,7 +1092,7 @@ pub mod commands {
 				if marked == 0 {
 					String::default()
 				} else {
-					format!(" {}", marked)
+					format!(" {marked}")
 				},
 				key_config.get_hint(key_config.keys.stash_drop),
 			),
@@ -1051,6 +1157,18 @@ pub mod commands {
 				key_config.get_hint(key_config.keys.blame),
 			),
 			"open blame view of selected file",
+			CMD_GROUP_GENERAL,
+		)
+	}
+	pub fn open_file_history(
+		key_config: &SharedKeyConfig,
+	) -> CommandText {
+		CommandText::new(
+			format!(
+				"History [{}]",
+				key_config.get_hint(key_config.keys.file_history),
+			),
+			"open history of selected file",
 			CMD_GROUP_LOG,
 		)
 	}
