@@ -1,9 +1,3 @@
-#![allow(
-	dead_code,
-	clippy::missing_const_for_fn,
-	clippy::unused_self
-)]
-
 use std::path::PathBuf;
 
 use crate::{
@@ -17,14 +11,15 @@ use crate::{
 	AsyncAppNotification, AsyncNotification,
 };
 use anyhow::Result;
-use asyncgit::sync::{self, RepoPathRef};
+use asyncgit::{
+	sync::{self, RepoPathRef},
+	AsyncGitNotification,
+};
 use crossbeam_channel::Sender;
 
 pub struct FilesTab {
 	repo: RepoPathRef,
 	visible: bool,
-	theme: SharedTheme,
-	key_config: SharedKeyConfig,
 	files: RevisionFilesComponent,
 }
 
@@ -33,6 +28,7 @@ impl FilesTab {
 	pub fn new(
 		repo: RepoPathRef,
 		sender: &Sender<AsyncAppNotification>,
+		sender_git: Sender<AsyncGitNotification>,
 		queue: &Queue,
 		theme: SharedTheme,
 		key_config: SharedKeyConfig,
@@ -43,11 +39,10 @@ impl FilesTab {
 				repo.clone(),
 				queue,
 				sender,
-				theme.clone(),
-				key_config.clone(),
+				sender_git,
+				theme,
+				key_config,
 			),
-			theme,
-			key_config,
 			repo,
 		}
 	}
@@ -69,10 +64,15 @@ impl FilesTab {
 	}
 
 	///
-	pub fn update_async(&mut self, ev: AsyncNotification) {
+	pub fn update_async(
+		&mut self,
+		ev: AsyncNotification,
+	) -> Result<()> {
 		if self.is_visible() {
-			self.files.update(ev);
+			self.files.update(ev)?;
 		}
+
+		Ok(())
 	}
 
 	pub fn file_finder_update(&mut self, file: &Option<PathBuf>) {
@@ -108,7 +108,7 @@ impl Component for FilesTab {
 
 	fn event(
 		&mut self,
-		ev: crossterm::event::Event,
+		ev: &crossterm::event::Event,
 	) -> Result<EventState> {
 		if self.visible {
 			return self.files.event(ev);
