@@ -2,19 +2,20 @@ use crate::{
 	components::{
 		visibility_blocking, CommandBlocking, CommandInfo,
 		CommitDetailsComponent, CommitList, Component,
-		DrawableComponent, EventState,
+		DrawableComponent, EventState, FileTreeOpen,
+		InspectCommitOpen,
 	},
-	keys::SharedKeyConfig,
-	queue::{InternalEvent, Queue},
+	keys::{key_match, SharedKeyConfig},
+	queue::{InternalEvent, Queue, StackablePopupOpen},
 	strings, try_or_popup,
 	ui::style::SharedTheme,
 };
 use anyhow::Result;
 use asyncgit::{
-	cached,
+	asyncjob::AsyncSingleJob,
 	sync::{self, CommitId, RepoPathRef},
-	AsyncGitNotification, AsyncLog, AsyncTags, CommitFilesParams,
-	FetchStatus,
+	AsyncBranchesJob, AsyncGitNotification, AsyncLog, AsyncTags,
+	CommitFilesParams, FetchStatus,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
@@ -35,9 +36,9 @@ pub struct Revlog {
 	list: CommitList,
 	git_log: AsyncLog,
 	git_tags: AsyncTags,
+	git_branches: AsyncSingleJob<AsyncBranchesJob>,
 	queue: Queue,
 	visible: bool,
-	branch_name: cached::BranchName,
 	key_config: SharedKeyConfig,
 }
 
@@ -63,6 +64,7 @@ impl Revlog {
 			list: CommitList::new(
 				&strings::log_title(&key_config),
 				theme,
+				queue.clone(),
 				key_config.clone(),
 			),
 			git_log: AsyncLog::new(
@@ -71,8 +73,8 @@ impl Revlog {
 				None,
 			),
 			git_tags: AsyncTags::new(repo.borrow().clone(), sender),
+			git_branches: AsyncSingleJob::new(sender.clone()),
 			visible: false,
-			branch_name: cached::BranchName::new(repo.clone()),
 			key_config,
 		}
 	}
@@ -81,6 +83,7 @@ impl Revlog {
 	pub fn any_work_pending(&self) -> bool {
 		self.git_log.is_pending()
 			|| self.git_tags.is_pending()
+			|| self.git_branches.is_pending()
 			|| self.commit_details.any_work_pending()
 	}
 
@@ -102,17 +105,13 @@ impl Revlog {
 
 			self.git_tags.request(Duration::from_secs(3), false)?;
 
-			self.list.set_branch(
-				self.branch_name.lookup().map(Some).unwrap_or(None),
-			);
-
 			if self.commit_details.is_visible() {
 				let commit = self.selected_commit();
 				let tags = self.selected_commit_tags(&commit);
 
 				self.commit_details.set_commits(
 					commit.map(CommitFilesParams::from),
-					tags,
+					&tags,
 				)?;
 			}
 		}
@@ -135,6 +134,17 @@ impl Revlog {
 						self.update()?;
 					}
 				}
+				AsyncGitNotification::Branches => {
+					if let Some(branches) =
+						self.git_branches.take_last()
+					{
+						if let Some(Ok(branches)) = branches.result()
+						{
+							self.list.set_branches(branches);
+							self.update()?;
+						}
+					}
+				}
 				_ => (),
 			}
 		}
@@ -163,11 +173,6 @@ impl Revlog {
 		self.list.selected_entry().map(|e| e.id)
 	}
 
-	fn copy_commit_hash(&self) -> Result<()> {
-		self.list.copy_entry_hash()?;
-		Ok(())
-	}
-
 	fn selected_commit_tags(
 		&self,
 		commit: &Option<CommitId>,
@@ -199,6 +204,17 @@ impl Revlog {
 
 		Ok(())
 	}
+
+	fn inspect_commit(&self) {
+		if let Some(commit_id) = self.selected_commit() {
+			let tags = self.selected_commit_tags(&Some(commit_id));
+			self.queue.push(InternalEvent::OpenPopup(
+				StackablePopupOpen::InspectCommit(
+					InspectCommitOpen::new_with_tags(commit_id, tags),
+				),
+			));
+		}
+	}
 }
 
 impl DrawableComponent for Revlog {
@@ -230,7 +246,9 @@ impl DrawableComponent for Revlog {
 }
 
 impl Component for Revlog {
-	fn event(&mut self, ev: Event) -> Result<EventState> {
+	//TODO: cleanup
+	#[allow(clippy::too_many_lines)]
+	fn event(&mut self, ev: &Event) -> Result<EventState> {
 		if self.visible {
 			let event_used = self.list.event(ev)?;
 
@@ -238,17 +256,24 @@ impl Component for Revlog {
 				self.update()?;
 				return Ok(EventState::Consumed);
 			} else if let Event::Key(k) = ev {
-				if k == self.key_config.keys.enter {
+				if key_match(k, self.key_config.keys.enter) {
 					self.commit_details.toggle_visible()?;
 					self.update()?;
 					return Ok(EventState::Consumed);
-				} else if k == self.key_config.keys.copy {
-					self.copy_commit_hash()?;
+				} else if key_match(k, self.key_config.keys.copy) {
+					try_or_popup!(
+						self,
+						strings::POPUP_FAIL_COPY,
+						self.list.copy_commit_hash()
+					);
 					return Ok(EventState::Consumed);
-				} else if k == self.key_config.keys.push {
+				} else if key_match(k, self.key_config.keys.push) {
 					self.queue.push(InternalEvent::PushTags);
 					return Ok(EventState::Consumed);
-				} else if k == self.key_config.keys.log_tag_commit {
+				} else if key_match(
+					k,
+					self.key_config.keys.log_tag_commit,
+				) {
 					return self.selected_commit().map_or(
 						Ok(EventState::NotConsumed),
 						|id| {
@@ -257,28 +282,23 @@ impl Component for Revlog {
 							Ok(EventState::Consumed)
 						},
 					);
-				} else if k == self.key_config.keys.focus_right
-					&& self.commit_details.is_visible()
+				} else if key_match(
+					k,
+					self.key_config.keys.focus_right,
+				) && self.commit_details.is_visible()
 				{
-					return self.selected_commit().map_or(
-						Ok(EventState::NotConsumed),
-						|id| {
-							self.queue.push(
-								InternalEvent::InspectCommit(
-									id,
-									self.selected_commit_tags(&Some(
-										id,
-									)),
-								),
-							);
-							Ok(EventState::Consumed)
-						},
-					);
-				} else if k == self.key_config.keys.select_branch {
+					self.inspect_commit();
+					return Ok(EventState::Consumed);
+				} else if key_match(
+					k,
+					self.key_config.keys.select_branch,
+				) {
 					self.queue.push(InternalEvent::SelectBranch);
 					return Ok(EventState::Consumed);
-				} else if k == self.key_config.keys.status_reset_item
-				{
+				} else if key_match(
+					k,
+					self.key_config.keys.status_reset_item,
+				) {
 					try_or_popup!(
 						self,
 						"revert error:",
@@ -286,40 +306,53 @@ impl Component for Revlog {
 					);
 
 					return Ok(EventState::Consumed);
-				} else if k == self.key_config.keys.open_file_tree {
+				} else if key_match(
+					k,
+					self.key_config.keys.open_file_tree,
+				) {
 					return self.selected_commit().map_or(
 						Ok(EventState::NotConsumed),
 						|id| {
 							self.queue.push(
-								InternalEvent::OpenFileTree(id),
+								InternalEvent::OpenPopup(
+									StackablePopupOpen::FileTree(
+										FileTreeOpen::new(id),
+									),
+								),
 							);
 							Ok(EventState::Consumed)
 						},
 					);
-				} else if k == self.key_config.keys.tags {
+				} else if key_match(k, self.key_config.keys.tags) {
 					self.queue.push(InternalEvent::Tags);
 					return Ok(EventState::Consumed);
-				} else if k == self.key_config.keys.compare_commits
-					&& self.list.marked_count() > 0
+				} else if key_match(
+					k,
+					self.key_config.keys.compare_commits,
+				) && self.list.marked_count() > 0
 				{
 					if self.list.marked_count() == 1 {
 						// compare against head
-						self.queue.push(
-							InternalEvent::CompareCommits(
-								self.list.marked()[0],
-								None,
+						self.queue.push(InternalEvent::OpenPopup(
+							StackablePopupOpen::CompareCommits(
+								InspectCommitOpen::new(
+									self.list.marked()[0].1,
+								),
 							),
-						);
+						));
 						return Ok(EventState::Consumed);
 					} else if self.list.marked_count() == 2 {
 						//compare two marked commits
 						let marked = self.list.marked();
-						self.queue.push(
-							InternalEvent::CompareCommits(
-								marked[0],
-								Some(marked[1]),
+						self.queue.push(InternalEvent::OpenPopup(
+							StackablePopupOpen::CompareCommits(
+								InspectCommitOpen {
+									commit_id: marked[0].1,
+									compare_id: Some(marked[1].1),
+									tags: None,
+								},
 							),
-						);
+						));
 						return Ok(EventState::Consumed);
 					}
 				}
@@ -424,6 +457,12 @@ impl Component for Revlog {
 	fn show(&mut self) -> Result<()> {
 		self.visible = true;
 		self.list.clear();
+
+		self.git_branches.spawn(AsyncBranchesJob::new(
+			self.repo.borrow().clone(),
+			true,
+		));
+
 		self.update()?;
 
 		Ok(())
