@@ -2,9 +2,10 @@ use crate::{
 	components::{
 		visibility_blocking, CommandBlocking, CommandInfo,
 		CommitList, Component, DrawableComponent, EventState,
+		InspectCommitOpen,
 	},
-	keys::SharedKeyConfig,
-	queue::{Action, InternalEvent, Queue},
+	keys::{key_match, SharedKeyConfig},
+	queue::{Action, InternalEvent, Queue, StackablePopupOpen},
 	strings,
 	ui::style::SharedTheme,
 };
@@ -33,6 +34,7 @@ impl StashList {
 			list: CommitList::new(
 				&strings::stashlist_title(&key_config),
 				theme,
+				queue.clone(),
 				key_config.clone(),
 			),
 			queue: queue.clone(),
@@ -67,7 +69,7 @@ impl StashList {
 				}
 				Err(e) => {
 					self.queue.push(InternalEvent::ShowErrorMsg(
-						format!("stash apply error:\n{}", e,),
+						format!("stash apply error:\n{e}",),
 					));
 				}
 			}
@@ -77,7 +79,7 @@ impl StashList {
 	fn drop_stash(&mut self) {
 		if self.list.marked_count() > 0 {
 			self.queue.push(InternalEvent::ConfirmAction(
-				Action::StashDrop(self.list.marked().to_vec()),
+				Action::StashDrop(self.list.marked_commits()),
 			));
 		} else if let Some(e) = self.list.selected_entry() {
 			self.queue.push(InternalEvent::ConfirmAction(
@@ -96,34 +98,50 @@ impl StashList {
 
 	fn inspect(&mut self) {
 		if let Some(e) = self.list.selected_entry() {
-			self.queue.push(InternalEvent::InspectCommit(e.id, None));
+			self.queue.push(InternalEvent::OpenPopup(
+				StackablePopupOpen::InspectCommit(
+					InspectCommitOpen::new(e.id),
+				),
+			));
 		}
 	}
 
 	/// Called when a pending stash action has been confirmed
 	pub fn action_confirmed(
+		&mut self,
 		repo: &RepoPath,
 		action: &Action,
 	) -> Result<()> {
 		match action {
-			Action::StashDrop(ids) => Self::drop(repo, ids)?,
-			Action::StashPop(id) => Self::pop(repo, *id)?,
+			Action::StashDrop(ids) => self.drop(repo, ids)?,
+			Action::StashPop(id) => self.pop(repo, *id)?,
 			_ => (),
 		};
 
 		Ok(())
 	}
 
-	fn drop(repo: &RepoPath, ids: &[CommitId]) -> Result<()> {
+	fn drop(
+		&mut self,
+		repo: &RepoPath,
+		ids: &[CommitId],
+	) -> Result<()> {
 		for id in ids {
 			sync::stash_drop(repo, *id)?;
 		}
 
+		self.list.clear_marked();
+		self.update()?;
+
 		Ok(())
 	}
 
-	fn pop(repo: &RepoPath, id: CommitId) -> Result<()> {
+	fn pop(&mut self, repo: &RepoPath, id: CommitId) -> Result<()> {
 		sync::stash_pop(repo, id)?;
+
+		self.list.clear_marked();
+		self.update()?;
+
 		Ok(())
 	}
 }
@@ -183,7 +201,7 @@ impl Component for StashList {
 
 	fn event(
 		&mut self,
-		ev: crossterm::event::Event,
+		ev: &crossterm::event::Event,
 	) -> Result<EventState> {
 		if self.is_visible() {
 			if self.list.event(ev)?.is_consumed() {
@@ -191,13 +209,22 @@ impl Component for StashList {
 			}
 
 			if let Event::Key(k) = ev {
-				if k == self.key_config.keys.enter {
+				if key_match(k, self.key_config.keys.enter) {
 					self.pop_stash();
-				} else if k == self.key_config.keys.stash_apply {
+				} else if key_match(
+					k,
+					self.key_config.keys.stash_apply,
+				) {
 					self.apply_stash();
-				} else if k == self.key_config.keys.stash_drop {
+				} else if key_match(
+					k,
+					self.key_config.keys.stash_drop,
+				) {
 					self.drop_stash();
-				} else if k == self.key_config.keys.stash_open {
+				} else if key_match(
+					k,
+					self.key_config.keys.stash_open,
+				) {
 					self.inspect();
 				}
 			}
