@@ -442,7 +442,7 @@ mod test {
 		assert_eq!(line_truncator, vec!["", "a"]);
 	}
 
-	/// Tests WordWrapper with words some of which exceed line length and some not.
+	/// Tests `WordWrapper` with words some of which exceed line length and some not.
 	#[test]
 	fn line_composer_word_wrapper_mixed_length() {
 		let width = 20;
@@ -471,11 +471,11 @@ mod test {
                     では、";
 		let (word_wrapper, word_wrapper_width) = run_composer(
 			Composer::WordWrapper { trim: true },
-			&text,
+			text,
 			width,
 		);
 		let (line_truncator, _) =
-			run_composer(Composer::LineTruncator, &text, width);
+			run_composer(Composer::LineTruncator, text, width);
 		assert_eq!(line_truncator, vec!["コンピュータ上で文字"]);
 		let wrapped = vec![
 			"コンピュータ上で文字",
@@ -592,7 +592,7 @@ mod test {
 		);
 
 		// Ensure that if the character was a regular space, it would be wrapped differently.
-		let text_space = text.replace("\u{00a0}", " ");
+		let text_space = text.replace('\u{00a0}', " ");
 		let (word_wrapper_space, _) = run_composer(
 			Composer::WordWrapper { trim: true },
 			&text_space,
